<template>
  <div
    :class="classes"
    class="divide-y divide-gray-900/10 rounded border border-gray-900/10"
  >
    <template
      v-for="group in Object.keys(permissions)"
      :key="group"
    >
      <Collapse>
        <template #default="{ collapsed, toggle }">
          <button
            type="button"
            class="flex w-full items-center justify-between p-3 text-left text-sm font-medium leading-6 text-gray-800 hover:cursor-pointer focus:outline-none focus:ring-2 focus:ring-primary-500"
            :class="{ 'border-b border-gray-900/10': !collapsed }"
            @click="toggle"
          >
            {{ $t(`app.admin.user_roles.permission_groups.${group}`) }}
            <ChevronDownRegularIcon
              aria-hidden="true"
              class="mx-2 h-4 w-4 fill-current transition-transform"
              :class="{ 'rotate-180': !collapsed }"
            />
          </button>
        </template>
        <template #content>
          <div class="space-y-2 py-4 pl-6 pr-3">
            <template
              v-for="permission in permissions[group]"
              :key="permission"
            >
              <FormSwitch
                :model-value="selectedPermissions.includes(permission)"
                :description="`app.admin.user_roles.permissions.${permission.replaceAll('.', '_')}.description`"
                :label="`app.admin.user_roles.permissions.${permission.replaceAll('.', '_')}.title`"
                @update:model-value="togglePermission(permission)"
              />
            </template>
          </div>
        </template>
      </Collapse>
    </template>
  </div>
</template>

<script lang="ts" setup>
import type { Permission } from "@/js/contexts/shared/models/Permission";
import { permissions } from "@/js/contexts/shared/models/Permission";
import type { ComputedRef } from "vue";
import { cloneDeep } from "lodash-es";
import FormSwitch from "@/js/contexts/form/ui/components/FormSwitch.vue";
import Collapse from "@/js/contexts/shared/ui/components/collapse/Collapse.vue";
import ChevronDownRegularIcon from "@/svg/icons/chevron-down-regular.svg?component";

const props = defineProps<{
  modelValue: Permission[] | null;
}>();

const emit = defineEmits<{
  "update:modelValue": [value: Permission[] | null];
}>();

const errors = inject<ComputedRef<boolean> | undefined>("errors", undefined);

const classes = computed(() => ({
  "border-red-500": errors?.value,
}));
const selectedPermissions = computed<Permission[]>(() => props.modelValue ?? []);

const togglePermission = (permission: Permission) => {
  const value = cloneDeep(props.modelValue ?? []);
  if (value.includes(permission)) {
    value.splice(value.indexOf(permission), 1);
  } else {
    value.push(permission);
  }
  emit("update:modelValue", value);
};
</script>
